// Commands.cpp : implementation file 
// 

#include "stdafx.h" 
#include "VSNetHelp.h" 
#include "Commands.h" 

#include "DlgSettings.h" 

/////////////////////////////////////////////////////////////////////////////
// CCommands

CCommands::CCommands() 
{ 
    m_pApplicationEventsObj = NULL; 
    m_pDebuggerEventsObj = NULL; 
} 

CCommands::~CCommands() 
{ 
} 

void CCommands::SetApplicationObject(IApplication* pApplication) 
{ 
    // This function assumes pApplication has already been AddRef'd for us, 
    // which CDSAddIn did in its QueryInterface call just before it called us. 
    m_pApplication = pApplication; 

    CComBSTR bstrVer; 
    HRESULT hr = m_pApplication->get_Version( & bstrVer); 
    ATLTRACE(L"==== %s ====\n", (BSTR)bstrVer); 

    // Create Application event handlers 
    XApplicationEventsObj::CreateInstance(&m_pApplicationEventsObj); 
    m_pApplicationEventsObj->AddRef(); 
    m_pApplicationEventsObj->Connect(m_pApplication); 
    m_pApplicationEventsObj->m_pCommands = this; 

    // Create Debugger event handler 
    CComPtr<IDispatch> pDebugger; 
    if (SUCCEEDED(m_pApplication->get_Debugger(&pDebugger)) && pDebugger != NULL) 
    { 
        XDebuggerEventsObj::CreateInstance(&m_pDebuggerEventsObj); 
        m_pDebuggerEventsObj->AddRef(); 
        m_pDebuggerEventsObj->Connect(pDebugger); 
        m_pDebuggerEventsObj->m_pCommands = this; 
    } 
} 

void CCommands::UnadviseFromEvents()
{ 
    HRESULT hr = NULL; 
    ATLASSERT (m_pApplicationEventsObj != NULL); 
    m_pApplicationEventsObj->Disconnect(m_pApplication);
    m_pApplicationEventsObj->Release();
    m_pApplicationEventsObj = NULL;

    if (m_pDebuggerEventsObj != NULL)
    {
        // Since we were able to connect to the Debugger events, we
        //  should be able to access the Debugger object again to
        //  unadvise from its events (thus the VERIFY below--see stdafx.h).
        CComPtr<IDispatch> pDebugger; 
        hr = m_pApplication->get_Debugger(&pDebugger); 
        ATLASSERT (SUCCEEDED(hr) && pDebugger != NULL);
        m_pDebuggerEventsObj->Disconnect(pDebugger);
        m_pDebuggerEventsObj->Release();
        m_pDebuggerEventsObj = NULL;
    }
}


/////////////////////////////////////////////////////////////////////////////
// Event handlers

// TODO: Fill out the implementation for those events you wish handle
//  Use m_pCommands->GetApplicationObject() to access the Developer
//  Studio Application object

// Application events

HRESULT CCommands::XApplicationEvents::BeforeBuildStart()
{ 
    ATLTRACE("XApplicationEvents::BeforeBuildStart \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::BuildFinish(long nNumErrors, long nNumWarnings)
{ 
    ATLTRACE("XApplicationEvents::BuildFinish \n"); 
    return S_OK;
} 

HRESULT CCommands::XApplicationEvents::BeforeApplicationShutDown()
{ 
    ATLTRACE("XApplicationEvents::BeforeApplicationShutDown \n"); 
    return S_OK;
} 

HRESULT CCommands::XApplicationEvents::DocumentOpen(IDispatch* theDocument)
{
    ATLTRACE("XApplicationEvents::DocumentOpen \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::BeforeDocumentClose(IDispatch* theDocument)
{
    ATLTRACE("XApplicationEvents::BeforeDocumentClose \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::DocumentSave(IDispatch* theDocument)
{ 
    ATLTRACE("XApplicationEvents::DocumentSave \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewDocument(IDispatch* theDocument)
{ 
    ATLTRACE("XApplicationEvents::NewDocument \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::WindowActivate(IDispatch* theWindow)
{ 
    ATLTRACE("XApplicationEvents::WindowActivate \n"); 
    return S_OK; 
} 

HRESULT CCommands::XApplicationEvents::WindowDeactivate(IDispatch* theWindow)
{ 
    ATLTRACE("XApplicationEvents::WindowDeactivate \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceOpen()
{ 
    ATLTRACE("XApplicationEvents::WorkspaceOpen \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceClose()
{ 
    ATLTRACE("XApplicationEvents::WorkspaceClose \n"); 
    return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewWorkspace()
{ 
    ATLTRACE("XApplicationEvents::NewWorkspace \n"); 
    return S_OK; 
} 

// Debugger event

HRESULT CCommands::XDebuggerEvents::BreakpointHit(IDispatch* pBreakpoint)
{ 
    ATLTRACE("XApplicationEvents::BreakpointHit \n"); 
    return S_OK;
} 


/////////////////////////////////////////////////////////////////////////////
// CCommands methods

STDMETHODIMP CCommands::VSNetHelpCommandMethod() 
{ 
    HRESULT hr = E_FAIL; 
    // TODO: Replace this with the actual code to execute this command
    //  Use m_pApplication to access the Developer Studio Application object,

    if (m_pApplication) 
    { 
        CRegKey regKey; 
        regKey.Create(HKEY_CURRENT_USER, g_pszMainKey); 
        DWORD dwHelpType = 0; 
        regKey.QueryValue(dwHelpType, g_pszHelpType); 

        hr = m_pApplication->EnableModeless(VARIANT_FALSE); ATLASSERT(SUCCEEDED(hr)); 
        CDlgSettings dlgSettings; 
        dlgSettings.m_nSelection = (eHelpSysType)dwHelpType; 
        if (IDOK == dlgSettings.DoModal()) { 
            // Write registry. 
            dwHelpType = (DWORD) dlgSettings.m_nSelection; 
            regKey.SetValue(dwHelpType, g_pszHelpType); 
        } 
        hr = m_pApplication->EnableModeless(VARIANT_TRUE); ATLASSERT(SUCCEEDED(hr)); 
    } 

    return S_OK;
}
