// AddInMod.cpp : implementation file
//

#include "stdafx.h"
#include "VSNetHelp.h"
#include "DSAddIn.h"
#include "Commands.h" 
#include <stdio.h> 

CComPtr<VsNetHelp::Help> g_spDotNetHelpSystem; 
eHelpSysType g_eCurrType = eTypeVC6; 

HRESULT (STDMETHODCALLTYPE * g_pFnOldKeywordSearch)(IUnknown * This, LPWSTR pszKeyword, long dwFlags, long dwReserved) = NULL;

HRESULT __stdcall MyKeywordSearch(IUnknown * This, LPWSTR pszKeyword, long dwFlags, long dwReserved)
{ 
    USES_CONVERSION; 
    HRESULT hr = E_FAIL; 

    const WCHAR * szCollection = NULL; 

    CRegKey regKey; 
    regKey.Create(HKEY_CURRENT_USER, g_pszMainKey); 
    DWORD dwHelpType = 0; 
    regKey.QueryValue(dwHelpType, g_pszHelpType); 

    switch((eHelpSysType)dwHelpType) 
    { 
    case eTypeVC6: 
        if (g_pFnOldKeywordSearch) { 
            return g_pFnOldKeywordSearch(This, pszKeyword, dwFlags, dwReserved); 
        } 
        break; 
    case eTypeVC71: 
        szCollection = T2CW(g_pszCollectionVC71); 
        break; 
    case eTypeVC8: 
        szCollection = T2CW(g_pszCollectionVC8); 
        break; 
    case eTypeVC9: 
        szCollection = T2CW(g_pszCollectionVC9); 
        break; 
    default: 
    case eTypeOnline: 
        { 
            WCHAR szURL[1024] = { 0 }; 
            lstrcpynW(szURL, T2CW(g_pszOnlineSearch), _countof(szURL)); 
            WCHAR *p = wcsstr(szURL, L"%1"); 
            if (p) 
            { 
                p[1] = L's'; 
                WCHAR szLink[1024]; 
                if (_snwprintf(szLink, _countof(szLink), szURL, pszKeyword) > 0) 
                { 
                    ShellExecuteW(NULL, L"open", szLink, NULL, NULL, SW_SHOWNORMAL); 
                    return S_OK; 
                } 
            } 
            return S_FALSE; 
        } 
        break; 
    } 

    if (g_eCurrType != (eHelpSysType)dwHelpType) 
    { 
        g_spDotNetHelpSystem.Release(); 
        hr = g_spDotNetHelpSystem.CoCreateInstance(L"DExplore.AppObj"); 
        if(FAILED(hr) || g_spDotNetHelpSystem == NULL) { 
            AtlMessageBox(NULL, IDS_CREATE_FAILED, IDS_VSNETHELP_LONGNAME, MB_OK | MB_ICONERROR); 
            return S_FALSE; 
        } 
        g_eCurrType = (eHelpSysType)dwHelpType; 
    } 

    if (g_spDotNetHelpSystem) { 
        g_spDotNetHelpSystem->SetCollection( CComBSTR(szCollection),  CComBSTR(L"")); 
        g_spDotNetHelpSystem->SyncIndex(pszKeyword, 1); 
        g_spDotNetHelpSystem->DisplayTopicFromF1Keyword(pszKeyword); 
    } else { 
        AtlMessageBox(NULL, IDS_CREATE_FAILED, IDS_VSNETHELP_LONGNAME, MB_OK | MB_ICONERROR); 
    } 

    return S_OK; 
}

// This is called when the user first loads the add-in, and on start-up
//  of each subsequent Developer Studio session
STDMETHODIMP CDSAddIn::OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,
                                    long dwCookie, VARIANT_BOOL* OnConnection)
{ 
    HRESULT hr = E_FAIL; 
    // Store info passed to us 
    CComPtr<IApplication> spApplication; 
    if (FAILED(pApp->QueryInterface(IID_IApplication, (void**) &spApplication))
        || spApplication == NULL)
    {
        *OnConnection = VARIANT_FALSE;
        return S_OK;
    }

    m_dwCookie = dwCookie; 

    // Create command dispatch, send info back to DevStudio
    CCommandsObj::CreateInstance(&m_pCommands);
    m_pCommands->AddRef();

    // The QueryInterface above AddRef'd the Application object.  It will
    //  be Release'd in CCommand's destructor.
    m_pCommands->SetApplicationObject(spApplication);

    hr = spApplication->SetAddInInfo((long) _Module.m_hInst, 
        (LPDISPATCH) m_pCommands, IDR_TOOLBAR_MEDIUM, IDR_TOOLBAR_LARGE, m_dwCookie); 
    ATLASSERT(SUCCEEDED(hr)); 

    LPCTSTR szCommand = _T("VSNetHelpCommand "); 
    VARIANT_BOOL bRet = VARIANT_FALSE; 
    CComBSTR strCmdString( szCommand ); 
    TCHAR szTmp[MAX_PATH] = { 0 }; 
    ::LoadString(_Module.m_hInst, IDS_CMD_STRING, szTmp, _countof(szTmp)); 
    strCmdString += szTmp;
    CComBSTR bszCmdString(strCmdString);
    CComBSTR bszMethod(_T("VSNetHelpCommandMethod")); 
    CComBSTR bszCmdName(szCommand); 
    hr = spApplication->AddCommand(bszCmdString, bszMethod, 0, m_dwCookie, &bRet); 
    ATLASSERT(SUCCEEDED(hr)); 
    if (bRet == VARIANT_FALSE) 
    { 
        // AddCommand failed because a command with this name already
        //  exists.  You may try adding your command under a different name.
        //  Or, you can fail to load as we will do here.
        *OnConnection = VARIANT_FALSE;
        return S_OK;
    } 

    if (bFirstTime == VARIANT_TRUE)
    { 
        hr = spApplication->AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie); 
        ATLASSERT(SUCCEEDED(hr)); 
    } 

    *OnConnection = VARIANT_TRUE; 

    //========================================================================== 
    CComPtr<Vs6Help::IVsHelpSystem> spVc6HelpSys; 
    hr = spVc6HelpSys.CoCreateInstance(__uuidof(Vs6Help::VsHelpServices)); 
    if (FAILED(hr)) return hr; 

    hr = spVc6HelpSys->QueryInterface(__uuidof(Vs6Help::IVsHelpSystem), (void **) &m_pRawHelp ); 
    if (FAILED(hr)) return hr; 

    ATLASSERT(m_pRawHelp); 

    ATLTRACE("m_pRawHelp = %x\n", m_pRawHelp); 
    ATLTRACE("lpVtbl = %x\n", m_pRawHelp->lpVtbl); 
    ATLTRACE("KeywordSearch = %x\n", m_pRawHelp->lpVtbl->KeywordSearch); 

    g_pFnOldKeywordSearch = m_pRawHelp->lpVtbl->KeywordSearch; 

    DWORD dwOldProtect = 0; 
    if (VirtualProtect(m_pRawHelp->lpVtbl, sizeof(_IVsHelpSystemVtbl), PAGE_READWRITE, &dwOldProtect)) 
        m_pRawHelp->lpVtbl->KeywordSearch = MyKeywordSearch; 

    return S_OK; 
}

// This is called on shut-down, and also when the user unloads the add-in
STDMETHODIMP CDSAddIn::OnDisconnection(VARIANT_BOOL bLastTime)
{ 
    m_pCommands->UnadviseFromEvents(); 
    m_pCommands->Release();
    m_pCommands = NULL;

    if (m_pRawHelp && m_pRawHelp->lpVtbl->KeywordSearch == MyKeywordSearch)
    { 
        m_pRawHelp->lpVtbl->KeywordSearch = g_pFnOldKeywordSearch; 
        m_pRawHelp->lpVtbl->Release((IUnknown *)m_pRawHelp); 
        m_pRawHelp = NULL; 
    } 

    g_spDotNetHelpSystem.Release(); 

    return S_OK;
}
